import { Http, Loader } from '@zeedhi/core';
import { InstanceService } from '@/services/InstanceService';
import { DialogServiceCustom } from '@/services/DialogServiceCustom';
import { DialogService, LoadingService, Modal, ModalService } from '@zeedhi/common';
import { AlertServiceCustom } from '@/services/AlertServiceCustom';

export class DMTabIssueEditController {
	public demandManagementController = Loader.getInstance('DemandManagementController');

	public defaultTimeoutTimer: any;

	public isCallTypeS() {
		return this.demandManagementController.callType === 'S';
	}

	public getFormEditIssue() {
		return InstanceService.getInstanceZdForm('formDemandManagementI');
	}

	/** Eventos de fields */

	public async onChangeClientEditCallModal(params: any) {
		if (params.value) {
			const screenNameInstance = InstanceService.getInstanceSelectTree('screenNameEditI');
			screenNameInstance.nodes = [];
			if (screenNameInstance.datasource) {
				screenNameInstance.datasource.filter.CDCLIENTE = params.value;
				try {
					await screenNameInstance.datasource.get();
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
				}
			}
		}
	}

	public async onChangeProductEditCallModal(params: any) {
		if (params.value) {
			const screenNameInstance = InstanceService.getInstanceSelectTree('screenNameEditI');
			screenNameInstance.nodes = [];
			if (screenNameInstance.datasource) {
				screenNameInstance.datasource.filter.PRODUCT_ID = params.value?.PRODUCT_FILTER_ID;
				try {
					await screenNameInstance.datasource.get();
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
				}
			}
		}
	}

	public async onFocusProductEditCall() {
		const productInstance = InstanceService.getInstanceZdSelect('productEditI');
		delete productInstance.datasource.filter.CALL_RELATIONSHIP;
		delete productInstance.datasource.filter.CDPRODCOMP;

		await productInstance.datasource.get();
	}

	public async onChangeScreenNameEditCallModal(params: any) {
		this.resetValidationsEDIT();
		const versionIInstance = InstanceService.getInstanceZdSelect('versionEditI');
		if (params.component.value) {
			versionIInstance.datasource.filter.PRODUCT_ID = params.component.value.NREXTERNAL_PRODUCT_ID;
			try {
				await versionIInstance.datasource.get();
			} catch (e) {
				DialogServiceCustom.showErrorBackend(e);
			}
		} else {
			versionIInstance.value = '';
		}
	}

	public async saveEditIssue() { // SAVE EDIT
		const callFormInstance = this.getFormEditIssue();
		if (callFormInstance.validate()) {
			const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
			const customerInstance = InstanceService.getInstanceZdSelect('customerEditI');
			LoadingService.show();
			try {
				await Http.post('/productCallSave', {
					CALL_RELATIONSHIP: this.demandManagementController.callRelationship,
					ID: this.demandManagementController.callID,
					REFERENCE: callFormInstance.value.referenceEditI,
					CALL_LABELS: callFormInstance.value.labelsEditI,
					DESCRIPTION: callFormInstance.value.descriptionEditI,
					REPORTED_BY: callFormInstance.value.reportedByEditI,
					CDCLIENTE: customerInstance.selectedData.CDCLIENTE,
					PLATAFORMA: 'MOBILE',
					PROJECT_ID: callFormInstance.value.productEditI.PRODUCT_FILTER_ID,
					BOARD: callFormInstance.value.productInternalEditI?.PRODUCT_FILTER_ID,
					SGSISTEMA: null,
					TYPE: 'I',
					PRODUCT_VERSION: '0',
					PRODUCT_RELEASE: '0',
					MENU_ITEM_ID: callFormInstance.value.screenNameEditI?.ID,
					MENU_VERSION_ID: callFormInstance.value.versionEditI,
					VERSION_DEV_ID: callFormInstance.value.devVersionEditI,
					CLASSIFICATION: callFormInstance.value.categoryEditI,
					APPOINTMENT_DATE: callFormInstance.value.appointmentDateEditI,
				});
				AlertServiceCustom.defaultSaveSuccessAlert();
				await callGridInstance.datasource.get();
			} catch (e) {
				LoadingService.hide();
				DialogServiceCustom.showErrorBackend(e);
				throw e;
			}
			LoadingService.hide();
		} else {
			DialogServiceCustom.messageErrorValidate();
		}
	}

	public resetValidationsEDIT() {
		const callFormInstance = this.getFormEditIssue();
		callFormInstance.resetValidation();
	}

	public resetInputtedValuesByClientEditField() {
		const callFormInstance = this.getFormEditIssue();
		callFormInstance.value.productEditI = '';
		callFormInstance.value.screenNameEditI = '';
		callFormInstance.value.versionEditII = '';
		callFormInstance.value.categoryEditI = '';
		callFormInstance.value.devVersionEditI = '';
	}

	public resetInputtedValuesByProductEditField() {
		const callFormInstance = this.getFormEditIssue();
		callFormInstance.value.screenNameEditI = '';
		callFormInstance.value.versionEditII = '';
		callFormInstance.value.categoryEditI = '';
		callFormInstance.value.devVersionEditI = '';
	}

	public resetInputtedValuesEDIT() {
		const callFormInstance = this.getFormEditIssue();
		callFormInstance.value.screenNameEdit = '';
		callFormInstance.value.versionEditI = '';
		callFormInstance.value.versionEditS = '';
	}

    private modalDescriptionI!: Modal;

    public async openModalDescriptionI() {
        LoadingService.show();
        this.modalDescriptionI = await ModalService.createFromJsonPost('', { jsonPath: 'demandManagement/modal/descriptionICallModal' });
        await this.modalDescriptionI.show();
        const formDescriptionDemandManagementI = InstanceService.getInstanceZdForm('formDescriptionDemandManagementI');
        const formDemandManagementI = InstanceService.getInstanceZdForm('formDemandManagementI');
        formDescriptionDemandManagementI.value.descriptionI = formDemandManagementI.value.descriptionEditI;
        LoadingService.hide();
    }

    public closeModalDescriptionICall() {
        this.modalDescriptionI.hide();
        this.modalDescriptionI.destroy();
    }

    public saveDescriptionICall() {
        const formDescriptionDemandManagementI = InstanceService.getInstanceZdForm('formDescriptionDemandManagementI');
        const formDemandManagementI = InstanceService.getInstanceZdForm('formDemandManagementI');
        formDemandManagementI.value.descriptionEditI = formDescriptionDemandManagementI.value.descriptionI;
        DialogService.show({
            name: 'dialog-save-form-call-i',
            type: 'warning',
            title: 'Atenção',
            text: 'Deseja realmente salvar todas as alterações do chamado?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.saveEditIssue();
                            DialogService.hide();
                            this.closeModalDescriptionICall();
                        },
                    },
                },
            ],
        });
    }
}
